class Section_mi:
	"""
	A class for calculation of I section properties for bending calculations

	...

	Attributes
	----------
	x1 : float
		top flange with of I beam in m
	y1 : float
		top flange thickness of I beam in m
	x2 : float
		web thickness of I beam in m
	y2 : float
		web height of I beam in m
	x3 : float
		bottom flange with of I beam in m
	y3 : float
		bottom flange thickness of I beam in m
	x4 : float
		top reinforced plate thickness of I beam in m
	y4 : float
		top reinforced plate height of I beam in m
	x5 : float
		bottom reinforced plate thickness of I beam in m
	y5 : float
		bottom reinforced plate height of I beam in m

	Methods
	-------
	get_centroid():
		Returns the centroid of section from bottom in m
	get_section_area():
		Returns the section area in m2
	get_inertia():
		Returns the section inertia in m4
	"""
	def __init__(self, x1, y1, x2, y2, x3, y3, x4, y4, x5, y5):
		"""
        Constructs all the necessary attributes for the Section_mi object.

        Attributes
		----------
		x1 : float
			top flange with of I beam in m
		y1 : float
			top flange thickness of I beam in m
		x2 : float
			web thickness of I beam in m
		y2 : float
			web height of I beam in m
		x3 : float
			bottom flange with of I beam in m
		y3 : float
			bottom flange thickness of I beam in m
		x4 : float
			top reinforced plate thickness of I beam in m
		y4 : float
			top reinforced plate height of I beam in m
		x5 : float
			bottom reinforced plate thickness of I beam in m
		y5 : float
			bottom reinforced plate height of I beam in m
		"""
		self.x1 = x1
		self.y1 = y1
		self.x2 = x2
		self.y2 = y2
		self.x3 = x3
		self.y3 = y3
		self.x4 = x4
		self.y4 = y4
		self.x5 = x5
		self.y5 = y5

		self.a1 = self.x1 * self.y1
		self.a2 = self.x2 * self.y2
		self.a3 = self.x3 * self.y3
		self.a4 = self.x4 * self.y4
		self.a5 = self.x5 * self.y5

		self.at = self.a1 + self.a2 + self.a3 + self.a4 + self.a5

		c1 = self.y2 + self.y3 + 0.5 * self.y1
		c2 = self.y3 + 0.5 * self.y2
		c3 = 0.5 * self.y3
		c4 = self.y2 + self.y3 - 0.5 * self.y4
		c5 = self.y3 + 0.5 * self.y5

		self.c = (self.a1 * c1 + self.a2 * c2 + self.a3 * c3 + self.a4 * c4 + 
			self.a5 * c5) / self.at

		self.case = 0
		self.inertia = 0


		if self.c >= self.y2 + self.y3:
			self.case = 1
			self.inertia = self.get_inertia_case1()
		elif self.c < (self.y2 + self.y3) and self.c >= (self.y2 + self.y3 - self.y4):
			self.case = 2
			self.inertia = self.get_inertia_case2()
		elif self.c < (self.y2 + self.y3 - self.y4) and self.c >= (self.y3 + self.y5):
			self.case = 3
			self.inertia = self.get_inertia_case3()
		elif self.c < (self.y3 + self.y5) and self.c >= self.y3:
			self.case = 4
			self.inertia = self.get_inertia_case4()
		elif self.c < self.y3:
			self.case = 5
			self.inertia = self.get_inertia_case5()

	def get_centroid(self):
		"""
		Returns the centroid of section from bottom in m

		Parameters
		----------

		Returns
		-------
		float
		Returns the centroid of section from bottom in m
		"""
		return self.c

	def get_case(self):
		return self.case

	def get_section_area(self):
		"""
		Returns the section area in m2

		Parameters
		----------

		Returns
		-------
		float
		Returns the section area in m2
		"""
		return self.at

	def get_inertia_case1(self):
		i = 0
		yu = self.y1 + self.y2 + self.y3 - self.c
		yl = self.c - self.y3 - self.y2
		ci1u = 0.5 * yu
		ci1l = 0.5 * yl
		ci2 = 0.5 * self.y2 + yl
		ci3 = 0.5 * self.y3 + self.y2 + yl
		ci4 = 0.5 * self.y4 + yl
		ci5 = self.y2 + yl - 0.5 * self.y5

		i1u = self.x1 * yu**3 / 12
		i1l = self.x1 * yl**3 / 12
		i2 = self.x2 * self.y2**3 / 12
		i3 = self.x3 * self.y3**3 / 12
		i4 = self.x4 * self.y4**3 / 12
		i5 = self.x5 * self.y5**3 / 12

		a1u = self.x1 * yu
		a1l = self.x1 * yl

		i = (i1u + a1u * ci1u**2) + (i1l + a1l * ci1l**2) + (i2 + self.a2 * ci2**2) + \
			(i3 + self.a3 * ci3**2) + (i4 + self.a4 * ci4**2) + (i5 + self.a5 * ci5**2)

		return i

	def get_inertia_case2(self):
		i = 0
		yu = self.y2 + self.y3 - self.c
		yl = self.c - self.y3
		yl2=self.y4-yu

		ci1 = 0.5 * self.y1+yu
		ci2u = 0.5 * yu
		ci2l = 0.5 * yl
		ci3 = 0.5 * self.y3 + yl
		ci4u = 0.5 * yu
		ci4l = 0.5*yl2
		ci5 = yl - 0.5 * self.y5

		i1 = self.x1 * self.y1**3 / 12
		i2u = self.x2 * yu**3 / 12
		i2l = self.x2 * yl**3 / 12
		i3 = self.x3 * self.y3**3 / 12
		i4u = self.x4 * yu**3 / 12
		i4l = self.x4 * yl2**3 / 12
		i5 = self.x5 * self.y5**3 / 12

		a2u = self.x2 * yu
		a2l = self.x2 * yl
		a4u = self.x4 * yu
		a4l = self.x4 * yl2

		i = (i1 + self.a1 * ci1**2) + (i2u + a2u * ci2u**2) + (i2l + a2l * ci2l**2) + \
			(i3 + self.a3 * ci3**2) + (i4u + a4u * ci4u**2) + (i4l + a4l * ci4l**2) + \
			(i5 + self.a5 * ci5**2)

		return i

	def get_inertia_case3(self):
		i = 0
		yu = self.y2 + self.y3 - self.c
		yl = self.c - self.y3

		ci1 = 0.5 * self.y1 + yu
		ci2u = 0.5 * yu
		ci2l = 0.5 * yl
		ci3 = self.c - 0.5 * self.y3
		ci4 = yu - 0.5 * self.y4
		ci5 = yl - 0.5 * self.y5

		i1 = self.x1 * self.y1**3 / 12
		i2u = self.x2 * yu**3 / 12
		i2l = self.x2 * yl**3 / 12
		i3 = self.x3 * self.y3**3 / 12
		i4 = self.x4 * self.y4**3 / 12
		i5 = self.x5 * self.y5**3 / 12

		a2u = self.x2 * yu
		a2l = self.x2 * yl

		i = (i1 + self.a1 * ci1**2) + (i2u + a2u * ci2u**2) + (i2l + a2l * ci2l**2) + \
			(i3 + self.a3 * ci3**2) + (i4 + self.a4 * ci4**2) + (i5 + self.a5 * ci5**2)

		return i

	def get_inertia_case4(self):
		i = 0
		yu = self.y2 + self.y3 - self.c
		yl = self.c - self.y3
		yu2 = self.y3 + self.y5 - self.c

		ci1 = 0.5 * self.y1 + yu
		ci2u = 0.5 * yu
		ci2l = 0.5 * yl
		ci3 = 0.5 * self.y3 + yl
		ci4 = yu - 0.5 * self.y4
		ci5u = 0.5 * yu2
		ci5l = 0.5 * yl

		i1 = self.x1 * self.y1**3 / 12
		i2u = self.x2 * yu**3 / 12
		i2l = self.x2 * yl**3 / 12
		i3 = self.x3 * self.y3**3 / 12
		i4 = self.x4 * self.y4**3 / 12
		i5u = self.x5 * yu2**3 / 12
		i5l = self.x5 * yl**3 / 12

		a2u = self.x2 * yu
		a2l = self.x2 * yl
		a5u = self.x5 * yu2
		a5l = self.x5 * yl

		i = (i1 + self.a1 * ci1**2) + (i2u + a2u * ci2u**2) + (i2l + a2l * ci2l**2) + \
			(i3 + self.a3 * ci3**2) + (i4 + self.a4 * ci4**2) + \
			(i5u + a5u * ci5u**2) + (i5l + a5l * ci5l**2)
			
		return i

	def get_inertia_case5(self):
		i = 0
		yu = self.y3 - self.c
		yl = self.c

		ci1 = 0.5 * self.y1 + self.y2 + yu
		ci2 = 0.5 * self.y2 + yu
		ci3u = 0.5 * yu
		ci3l = 0.5 * yl
		ci4 = yu + self.y2 - 0.5 * self.y4
		ci5 = 0.5 * self.y5 + yu

		i1 = self.x1 * self.y1**3 / 12
		i2 = self.x2 * self.y2**3 / 12
		i3u = self.x3 * yu**3 / 12
		i3l = self.x3 * yl**3 / 12
		i4 = self.x4 * self.y4**3 / 12
		i5 = self.x5 * self.y5**3 / 12

		a3u = self.x3 * yu
		a3l = self.x3 * yl

		i = (i1 + self.a1 * ci1**2) + (i2 + self.a2 * ci2**2) + \
			(i3u + a3u * ci3u**2) + (i3l + a3l * ci3l**2) + \
			(i4 + self.a4 * ci4**2) + (i5 + self.a5 * ci5**2)
		return i

	def get_inertia(self):
		"""
		Returns the section inertia in m4

		Parameters
		----------

		Returns
		-------
		float
		Returns the section inertia in m4
		"""
		return self.inertia


# my_sec = Section_mi(100, 10, 10, 100, 100, 10, 5, 25, 5, 25)
# my_sec = Section_mi(500, 50, 100, 2000, 500, 50, 0, 0, 0, 0)
# print(my_sec.get_centroid())
# print(my_sec.get_case())
# print(my_sec.get_inertia())
# print(my_sec.get_section_area())



import math

class ED:
	"""
	A class to represent a Eccentric Drive Mechanical Press

	...

	Attributes
	----------
	r : float
	    Eccentricity or half of Press Stroke in m
	l : float
	    Conrod length in m
	s : float
	    Rated distance in m
	f : float
	    Press force in N

	Methods
	-------
	get_alp_rad():
		Returns the rated angle of crank in radian
	get_beta_rad():
		Returns the rated angle of conrod in radian
	get_torque():
		Returns the rated torque at Eccentric Gear in Nm
	get_fbos(th2_rad):
		Returns the Slide distance FBOS (From Bottom Of Stroke) at given angle in m
	get_slide_vel(th2_rad, w2):
		Returns the Slide velocity (in m/s) at given angle and angular velocity of crank
	get_slide_acc(th2_rad, w2, alp2):
		Returns the Slide acceleration (in m/s2) at given angle, angular velocity and angular acceleration of crank
	get_f(th2_rad):
		Returns the Available force (in N) at given crank angle
	"""

	def __init__(self, r, l, s, f):
		"""
        Constructs all the necessary attributes for the ED (EccentricDrive) object.

        Parameters
        ----------
		r : float
		    Eccentricity or half of Press Stroke in m
		l : float
		    Conrod length in m
		s : float
		    Rated distance in m
		f : float
		    Press force in N
		"""
		self.r = r  # eccentricity in m
		self.l = l  # con rod length in m
		self.s = s  # rated distance in m
		self.f = f  # press force in N
		self.c1 = self.r + self.l - self.s
		self.alp_rad = math.acos((self.r**2 + self.c1**2 -self.l**2)/(2*self.r*self.c1))  # rated angle in rad
		self.alp_deg = self.alp_rad * 180 / math.pi
		self.beta_rad = math.acos((self.l**2 + self.c1**2 -self.r**2)/(2*self.l*self.c1))  # conrod rated angle 
		self.beta_deg = self.beta_rad * 180 / math.pi
		self.t = self.f * self.r * (math.sin(self.alp_rad) + math.cos(self.alp_rad)*math.tan(self.beta_rad))  # torque at EG in Nm

		"""
		Following code is for generating list for plotting graph
		It will be activated in future revision

		TH2_S_DEG = 90  # start crank angle in deg
		self.th2_deg_lst = []  # crank angle from tdc in deg
		self.f_lst = []  # press force in N
		self.fbos_lst = []  # fbos in m
		self.th2_deg_lst.clear()
		self.f_lst.clear()
		self.fbos_lst.clear()
		for x in range(TH2_S_DEG, 181):
		    self.th2_deg_lst.append(x)
		    this_th2_rad = x * math.pi / 180  # crank angle from tdc in rad
		    this_alp_rad = math.pi - this_th2_rad  # crank angle from BDC
		    this_beta_rad = math.asin(self.r * math.sin(this_th2_rad) / self.l)
		    this_fbos = self.r + self.l - (self.l * math.cos(this_beta_rad) - self.r * math.cos(this_th2_rad))
		    self.fbos_lst.append(this_fbos)
		    try:
		        this_f = self.t / (self.r * (math.sin(this_alp_rad) + math.cos(this_alp_rad)*math.tan(this_beta_rad)))  # force in N
		    except ZeroDivisionError:
		        this_f = self.f
		    
		    if this_f > self.f:
		        this_f = self.f

		    self.f_lst.append(this_f)
		"""

    
	def get_alp_rad(self):
		"""
		Returns the rated angle of crank in radian

		Parameters
		----------

		Returns
		-------
		float
		Returns the rated angle of crank in radian
		"""
		return self.alp_rad


	def get_beta_rad(self):
		"""
		Returns the rated angle of conrod in radian

		Parameters
		----------

		Returns
		-------
		float
		Returns the rated angle of conrod in radian
		"""
		return self.beta_rad

	def get_torque(self):
		"""
		Returns the rated torque at Eccentric Gear in Nm

		Parameters
		----------

		Returns
		-------
		float
		Returns the rated torque at Eccentric Gear in Nm
		"""
		return self.t

	def get_fbos(self, th2_rad):
		"""
		Returns the Slide distance FBOS (From Bottom Of Stroke) at given angle in m

		Parameters
		----------
		th2_rad : float
		    Crank angle from TDC in rad

		Returns
		-------
		float
		Returns the FBOS in m
		"""
		th3_rad = math.asin(self.r * math.sin(th2_rad) / self.l)  #conrod angle in rad
		fbos = self.l + self.r - (self.l * math.cos(th3_rad) - self.r * math.cos(th2_rad))
		return fbos  # in m

	def get_slide_vel(self, th2_rad, w2):
		"""
		Returns the Slide velocity (in m/s) at given angle and angular velocity of crank

		Parameters
		----------
		th2_rad : float
		    Crank angle from TDC in rad

		w2 : float
		    Angular velocity of crank in rad/s

		Returns
		-------
		float
		Returns the Slide velocity in m/s
		"""
		th3_rad = math.asin(self.r * math.sin(th2_rad) / self.l)  #conrod angle in rad
		w3 = self.r * math.cos(th2_rad) * w2 / (self.l * math.cos(th3_rad))
		slide_vel = self.r * w2 * math.sin(th2_rad) - self.l * w3 * math.sin(th3_rad)  # in m/s
		return slide_vel  # in m/s
	
	def get_slide_acc(self, th2_rad, w2, alp2):
		"""
		Returns the Slide acceleration (in m/s2) at given angle, angular velocity and angular acceleration of crank

		Parameters
		----------
		th2_rad : float
		    Crank angle from TDC in rad

		w2 : float
		    Angular velocity of crank in rad/s

		alp2 : float
		    Angular acceleration of crank in rad/s2

		Returns
		-------
		float
		Returns the Slide acceleration in m/s2
		"""
		th3_rad = math.asin(self.r * math.sin(th2_rad) / self.l)  # conrod angle in rad
		w3 = self.r * math.cos(th2_rad) * w2 / (self.l * math.cos(th3_rad))
		alp3 = (self.r * alp2 * math.cos(th2_rad) - self.r * w2**2 * math.sin(th2_rad) + self.l * w3**2 * math.sin(th3_rad)) / (self.l * math.cos(th3_rad))
		slide_acc = self.r * alp2 * math.sin(th2_rad) + self.r * w2**2 * math.cos(th2_rad) - self.l * alp3 * math.sin(th3_rad) - self.l * w3**2 * math.cos(th3_rad)
		return slide_acc  # in m/s2

	def get_f(self, th2_rad):
		"""
		Returns the Available force (in N) at given crank angle

		Parameters
		----------
		th2_rad : float
		    Crank angle from TDC in rad

		Returns
		-------
		float
		Returns the available press force in N
		"""
		th3_rad = math.asin(self.r * math.sin(th2_rad) / self.l)  # conrod angle in rad
		f_this = self.t * math.cos(th3_rad) / (self.r * math.sin((math.pi - th2_rad) + th3_rad))
		if f_this > self.f:
			f_this = self.f
		return f_this  # in N


import math
# this class has been validated with excel sheet

class Crown:
	"""
	A class for calculation of Crown of a mechanical press 
	with 2 or 4 suspensions

	...

	Attributes
	----------
	fr : float
		rated force of press in N
	l : float
		Tie rod center distance in m
	sus_cd : float
		Distance between suspensions in m
	y : float
		distance of farthest fiber from centroid in m
	i : float
		section inertia in m4
	x2 : float
		section web width in m
	y2 : float
		section web height in m
	e : float (Optional)
		Youngs modulus in N/m2
	g : float (Optional)
		shear modulus in N/m2

	Methods
	-------
	get_sb():
		Returns the stress due to bending in N/m2
	get_ss():
		Returns the stress due to shear in N/m2
	get_def_b():
		Returns the deflection due to bending in m
	get_def_s():
		Returns the deflection due to shear in m
	"""
	def __init__(self, fr, l, sus_cd, y, i, x2, y2, e = 21E10, g = 8E10):
		"""
        Constructs all the necessary attributes for the Crown object.

        Parameters
		----------
		fr : float
			rated force of press in N
		l : float
			Tie rod center distance in m
		sus_cd : float
			Distance between suspensions in m
		y : float
			distance of farthest fiber from centroid in m
		i : float
			section inertia in m4
		x2 : float
			section web width in m
		y2 : float
			section web height in m
		e : float
			Youngs modulus in N/m2
		g : float
			shear modulus in N/m2
		"""
		self.fr = fr  # press force in N
		self.l = l  # tie rod CD in m
		self.sus_cd = sus_cd  # suspension cd in m
		self.y = y  # distance of farthest fiber from centroid in m
		self.i = i  # section mi in m4
		self.x2 = x2  # section web width in m
		self.y2 = y2  # section web height in m
		self.e = e  # youngs modulus in N/m2
		self.g = g  # shear modulus in N/m2

		self.a = (self.l - self.sus_cd) / 2  # distance of suspension from tie rod in m
		self.f = self.fr / 2  # suspension force on one side in N
		self.bm = self.f * self.a  # bending moemnt at suspension (max) Nm
		self.v = self.f  # Shear force in N

		self.s_b = self.bm * self.y / self.i  # N/m2  stress in bending
		self.s_s = 1.5 * self.v / (self.x2 * self.y2)  # N/m2  stress in shear
		self.def_b = - self.f * self.a * (4 * self.a**2 - 3 * self.l**2) / \
		(24 * self.e * self.i)  # deflection due to bending in m
		self.def_s = 1.5 * self.a * self.v / (self.x2 * self.y2 * self.g)  
		# above is deflection due to shear in m

	def get_sb(self):
		"""
		Returns the stress due to bending in N/m2

		Parameters
		----------

		Returns
		-------
		float
		Returns the stress due to bending in N/m2
		"""
		return self.s_b  # N/m2  stress in bending

	def get_ss(self):
		"""
		Returns the stress due to shear in N/m2

		Parameters
		----------

		Returns
		-------
		float
		Returns the stress due to shear in N/m2
		"""
		return self.s_s  # N/m2  stress in shear

	def get_def_b(self):
		"""
		Returns the deflection due to bending in m

		Parameters
		----------

		Returns
		-------
		float
		Returns the deflection due to bending in m
		"""
		return self.def_b  # deflection due to bending in m

	def get_def_s(self):
		"""
		Returns the deflection due to shear in m

		Parameters
		----------

		Returns
		-------
		float
		Returns the deflection due to shear in m
		"""
		return self.def_s  # above is deflection due to shear in m

# def __init__(self, fr, l, sus_cd, y, i, x2, y2, e = 21E10, g = 8E10):
# c1 = Crown(12500000, 5, 3, 1.2, 0.4245, 0.2, 2)
# print("Bending stress (MPa): ", c1.get_sb() / 1000000)
# print("Shear stress (MPa): ", c1.get_ss() / 1000000)
# print("Def in bending (mm): ", c1.get_def_b() * 1000)
# print("Def in shear (mm): ", c1.get_def_s() * 1000)


import math

class LD:
	"""
	A class to represent a 6 Link Drive Mechanical Press

	...

	Attributes
	----------
	a : float
	    Eccentricity in m
	b : float
	    Ternary link length (rocker side) in m
	c : float
	    Rocker link length in m
	d : float
	    Rocker x distance from Eccentric Gear rotating center in m (+ve value only)
	f : float
	    Rocker y distance from Eccentric Gear rotating center in m (+ve value only)
	tht:float
		Obtuse angle between 2 sides of ternary link in radian
	g : float
	    Conrod length (connected to slide) in m
	h : float
	    Slide offset in m (+ve value if offset is away from rocker side)
	m : float
	    Ternary link length (conrod side) in m
	fr: float
	    Rated press force in N
	rd: float
	    Rated distance in m

	Methods
	-------
	get_stroke():
		Returns Slide stroke in m
	get_eg_torque():
		Returns the rated torque at Eccentric Gear in Nm
	get_th2_decideg_lst():
		Returns the list of crank angle for 1 complete rotation in resolution of 0.1 deg
	get_fbos_lst():
		Returns the list of fbos with respect to th2_decideg_lst
	get_vel_lst(w2):
		Returns the list of slide velocity with respect to th2_decideg_lst at given angular velocity of crank
	get_th2_tdc():
		Returns crank angle at TDC in deg
	get_th2_bdc():
		Returns crank angle at BDC in deg

	Notes
	-----
	
	Folowing sketch is for reference only.
	Actual sketch considered in the program is mirror of this.
	
	............f............
	
	                 o
	              / / \\
	O1          /  /   \\        .
	|         /   /     c        .
	|       b    /       \\      d
	a     /     /         \\     .
	|   /      /           O2    .
	| /       /
	o        /
	|       /
	|      /
	|     /
	m    /
	|   /
	|  /
	| /
	|/
	o
	|
	|
	|
	|
	g
	|
	|
	|
	|
	O3


	O1 is Eccentric gear rotation center
	O2 is rocker link pivot point
	O3 is slide connection point
	"""

	def __init__(self, a, b, c, d, f, tht, g, h, m, fr, rd):

		"""
        Constructs all the necessary attributes for the LD (6 Link Drive) object.

		Parameters
		----------
		a : float
		    Eccentricity in m
		b : float
		    Ternary link length (rocker side) in m
		c : float
		    Rocker link length in m
		d : float
		    Rocker x distance from Eccentric Gear rotating center in m (+ve value only)
		f : float
		    Rocker y distance from Eccentric Gear rotating center in m (+ve value only)
		tht:float
			Obtuse angle between 2 sides of ternary link in radian
		g : float
		    Conrod length (connected to slide) in m
		h : float
		    Slide offset in m (+ve value if offset is away from rocker side)
		m : float
		    Ternary link length (conrod side) in m
		fr: float
		    Rated press force in N
		rd: float
		    Rated distance in m
		

		Note
		----
		Positive X axis is considered zero degree crank angle.
		Counter clockwise direction is considered as positive for angle rotation.
		This is similar to cartesian coordinate system.
		When crank angle (th2...) = 0, slide in near mid stroke.
		if th2 increases from 0 deg, slide moves towards TDC.
		"""
		self.a = a  # eccentricity
		self.b = b  # ternary link length rocker side
		self.c = c  # rocker length
		self.d = d  # rocker x (+ve value only)
		self.f = f  # rocker y (+ve value only)
		
		self.tht = tht  # ternary link angle
		self.g = g  # conrod length
		self.h = h  # slide offset  (+ve value if offset is away from rocker side)
		self.m = m  # ternary link length conrod side

		self.fr = fr  # rated press force in N
		self.rd = rd  # rated distance of press in m

		self.k1 = d/a
		self.k2 = d/c
		self.k3 = (a**2 - b**2 + c**2 + d**2 + f**2)/(2 * a * c)
		self.k4 = f/c
		self.k5 = f/a

		self.th2_decideg_lst = []
		self.k_lst = []
		self.vel_lst = []

		self.w2 = 1  # crank ang vel in rad/s assumed as 1 rad/s

		for th2_decideg in range(3601):
			th2 = math.pi * (th2_decideg / 10) / 180  # crank angle in rad
			self.th2_decideg_lst.append(th2)

			A = math.cos(th2) + self.k1 + self.k3 + self.k2 * math.cos(th2) + self.k4 * math.sin(th2)
			B = -2 * math.sin(th2) - 2 * self.k5
			C = self.k3 + self.k2 * math.cos(th2) + self.k4 * math.sin(th2) - math.cos(th2) - self.k1

			th4 = 2 * math.atan((-B + math.pow(B**2 - 4 * A * C, 0.5)) / (2 * A))
			th3 = math.acos(c * math.cos(th4)/b - a * math.cos(th2) / b - d / b)
			th7 = th3 - (math.pi - tht)
			th8 = math.acos(a * math.cos(th2) / g - m * math.cos(th7) / g - h / g)
			k = - a * math.sin(th2) + m * math.sin(th7) + g * math.sin(th8)  # k is slide position from eg rotating center
			self.k_lst.append(k)


			w4 = self.w2 * (a/c) * (math.sin(th3 - th2) / math.sin(th3 - th4))
			w3 = c * w4 * math.sin(th4) / (b * math.sin(th3)) - a * self.w2 * math.sin(th2) / (b * math.sin(th3))

			w7 = w3
			w8 = self.w2 * (a / g) * (math.sin(th2) / math.sin(th8)) - w7 * (m / g) * (math.sin(th7) / math.sin(th8)) 
			v_k = - a * self.w2 * math.cos(th2) + m * w7 * math.cos(th7) + g * w8 * math.cos(th8)
			self.vel_lst.append(v_k)


		self.min_k = min(self.k_lst)  # min distance of slide from eg center
		self.max_k = max(self.k_lst)  # max distance of slide from eg center
		self.stroke = self.max_k - self.min_k  # stroke of slide in m

	def get_stroke(self):
		"""
		Returns the stroke in m

		Parameters
		----------

		Returns
		-------
		float
		Returns the stroke in m
		"""
		return self.stroke

	def get_eg_torque(self):
		"""
		Returns the Eccentric gear torque in Nm

		Parameters
		----------

		Returns
		-------
		float
		Returns the Eccentric gear torque in Nm
		"""
		k_rd = self.max_k - self.rd  # slide position from eg center at rated distance in m
		x_at_rd = 0  # index of list of fbos at rated distance
		for x in range(3601):
			this_k = self.k_lst[x]
			this_vel = self.vel_lst[x]
			if this_vel > 0:
				if k_rd < this_k:
					x_at_rd = x - 1
					break
		vel_at_rd = self.vel_lst[x_at_rd]
		t_eg = self.fr * vel_at_rd / self.w2  # eg torque in Nm
		return t_eg
		
	def get_th2_decideg_lst(self):
		"""
		Returns the list containing crank angle

		Parameters
		----------

		Returns
		-------
		float
		Returns the list containing crank angle (in radian) with least count of 0.1 deg which is equal to 0.00174 radian
		"""
		return self.th2_decideg_lst  # mention about sign and index, CA = 0 in pos x axis

	def get_fbos_lst(self):
		"""
		Returns the FBOS list at each 0.1 degree in m

		Parameters
		----------

		Returns
		-------
		float
		Returns the FBOS list at each 0.1 degree in m
		"""
		fbos_lst = []
		for x in self.k_lst:
			fbos_lst.append(self.max_k - x)
		return fbos_lst

	def get_vel_lst(self, w2):
		"""
		Returns the SLide velocity list at each 0.1 degree in m/s

		Parameters
		----------

		Returns
		-------
		float
		Returns the SLide velocity list at each 0.1 degree in m/s
		"""
		vel_at_w2_lst = []
		for x in self.vel_lst:
			vel_at_w2_lst.append(x * w2)
		return vel_at_w2_lst  # mention about sign and index, CA = 0 in pos x axis

	def get_th2_tdc(self):
		"""
		Returns the crank angle at TDC in degree.
		Positive X axis is considered zero degree crank angle.
		Counter clockwise direction is considered as positive for angle rotation.
		This is similar to cartesian coordinate system.

		Parameters
		----------

		Returns
		-------
		float
		Returns the crank angle at TDC in degree
		"""
		x_at_tdc = 0  # index of list of fbos at tdc - init only
		for x in range(3601):
			this_k = self.k_lst[x]
			if this_k == self.min_k:
				x_at_tdc = x 
				break
		return x_at_tdc / 10  # in deg

	def get_th2_bdc(self):
		"""
		Returns the crank angle at BDC in degree.
		Positive X axis is considered zero degree crank angle.
		Counter clockwise direction is considered as positive for angle rotation.
		This is similar to cartesian coordinate system.

		Parameters
		----------

		Returns
		-------
		float
		Returns the crank angle at BDC in degree
		"""
		x_at_bdc = 0  # index of list of fbos at bdc - init only
		for x in range(3601):
			this_k = self.k_lst[x]
			if this_k == self.max_k:
				x_at_bdc = x 
				break
		return x_at_bdc / 10  # in deg
